@@ -7,18 +7,15 @@ import pytest
 from scipy import linalg
 
 from sklearn.model_selection import train_test_split
-from sklearn.utils.testing import assert_allclose
-from sklearn.utils.testing import assert_array_almost_equal
-from sklearn.utils.testing import assert_equal
-from sklearn.utils.testing import assert_less
-from sklearn.utils.testing import assert_greater
-from sklearn.utils.testing import assert_raises
-from sklearn.utils.testing import ignore_warnings
-from sklearn.utils.testing import assert_warns
-from sklearn.utils.testing import TempMemmap
+from sklearn.utils._testing import assert_allclose
+from sklearn.utils._testing import assert_array_almost_equal
+from sklearn.utils._testing import assert_raises
+from sklearn.utils._testing import ignore_warnings
+from sklearn.utils._testing import assert_warns
+from sklearn.utils._testing import TempMemmap
 from sklearn.exceptions import ConvergenceWarning
 from sklearn import linear_model, datasets
-from sklearn.linear_model.least_angle import _lars_path_residues, LassoLarsIC
+from sklearn.linear_model._least_angle import _lars_path_residues, LassoLarsIC
 
 # TODO: use another dataset that has multiple drops
 diabetes = datasets.load_diabetes()
@@ -78,7 +75,7 @@ def test_simple_precomputed():
 
 
 def _assert_same_lars_path_result(output1, output2):
-    assert_equal(len(output1), len(output2))
+    assert len(output1) == len(output2)
     for o1, o2 in zip(output1, output2):
         assert_allclose(o1, o2)
 
@@ -150,7 +147,7 @@ def test_collinearity():
     _, _, coef_path_ = f(linear_model.lars_path)(X, y, alpha_min=0.01)
     assert not np.isnan(coef_path_).any()
     residual = np.dot(X, coef_path_[:, -1]) - y
-    assert_less((residual ** 2).sum(), 1.)  # just make sure it's bounded
+    assert (residual ** 2).sum() < 1.  # just make sure it's bounded
 
     n_samples = 10
     X = rng.rand(n_samples, 5)
@@ -199,7 +196,6 @@ def test_no_path_all_precomputed():
     assert alpha_ == alphas_[-1]
 
 
-@pytest.mark.filterwarnings('ignore: The default value of cv')  # 0.22
 @pytest.mark.parametrize(
         'classifier',
         [linear_model.Lars, linear_model.LarsCV, linear_model.LassoLarsIC])
@@ -247,7 +243,7 @@ def test_rank_deficient_design():
         coef_cd_ = coord_descent.fit(X, y).coef_
         obj_cd = ((1. / (2. * 3.)) * linalg.norm(y - np.dot(X, coef_cd_)) ** 2
                   + .1 * linalg.norm(coef_cd_, 1))
-        assert_less(obj_lars, obj_cd * (1. + 1e-8))
+        assert obj_lars < obj_cd * (1. + 1e-8)
 
 
 def test_lasso_lars_vs_lasso_cd():
@@ -263,7 +259,7 @@ def test_lasso_lars_vs_lasso_cd():
         lasso_cd.alpha = a
         lasso_cd.fit(X, y)
         error = linalg.norm(c - lasso_cd.coef_)
-        assert_less(error, 0.01)
+        assert error < 0.01
 
     # similar test, with the classifiers
     for alpha in np.linspace(1e-2, 1 - 1e-2, 20):
@@ -271,7 +267,7 @@ def test_lasso_lars_vs_lasso_cd():
         clf2 = linear_model.Lasso(alpha=alpha, tol=1e-8,
                                   normalize=False).fit(X, y)
         err = linalg.norm(clf1.coef_ - clf2.coef_)
-        assert_less(err, 1e-3)
+        assert err < 1e-3
 
     # same test, with normalized data
     X = diabetes.data
@@ -284,7 +280,7 @@ def test_lasso_lars_vs_lasso_cd():
         lasso_cd.alpha = a
         lasso_cd.fit(X, y)
         error = linalg.norm(c - lasso_cd.coef_)
-        assert_less(error, 0.01)
+        assert error < 0.01
 
 
 def test_lasso_lars_vs_lasso_cd_early_stopping():
@@ -300,18 +296,17 @@ def test_lasso_lars_vs_lasso_cd_early_stopping():
         lasso_cd.alpha = alphas[-1]
         lasso_cd.fit(X, y)
         error = linalg.norm(lasso_path[:, -1] - lasso_cd.coef_)
-        assert_less(error, 0.01)
+        assert error < 0.01
 
     # same test, with normalization
     for alpha_min in alphas_min:
         alphas, _, lasso_path = linear_model.lars_path(X, y, method='lasso',
                                                        alpha_min=alpha_min)
-        lasso_cd = linear_model.Lasso(fit_intercept=True, normalize=True,
-                                      tol=1e-8)
+        lasso_cd = linear_model.Lasso(normalize=True, tol=1e-8)
         lasso_cd.alpha = alphas[-1]
         lasso_cd.fit(X, y)
         error = linalg.norm(lasso_path[:, -1] - lasso_cd.coef_)
-        assert_less(error, 0.01)
+        assert error < 0.01
 
 
 def test_lasso_lars_path_length():
@@ -383,7 +378,7 @@ def test_lasso_lars_vs_lasso_cd_ill_conditioned2():
     cd_coef_ = coord_descent.fit(X, y).coef_
     cd_obj = objective_function(cd_coef_)
 
-    assert_less(lars_obj, cd_obj * (1. + 1e-8))
+    assert lars_obj < cd_obj * (1. + 1e-8)
 
 
 def test_lars_add_features():
@@ -400,10 +395,10 @@ def test_lars_add_features():
 def test_lars_n_nonzero_coefs(verbose=False):
     lars = linear_model.Lars(n_nonzero_coefs=6, verbose=verbose)
     lars.fit(X, y)
-    assert_equal(len(lars.coef_.nonzero()[0]), 6)
+    assert len(lars.coef_.nonzero()[0]) == 6
     # The path should be of length 6 + 1 in a Lars going down to 6
     # non-zero coefs
-    assert_equal(len(lars.alphas_), 7)
+    assert len(lars.alphas_) == 7
 
 
 @ignore_warnings
@@ -434,7 +429,6 @@ def test_multitarget():
             assert_array_almost_equal(Y_pred[:, k], y_pred)
 
 
-@pytest.mark.filterwarnings('ignore: The default value of cv')  # 0.22
 def test_lars_cv():
     # Test the LassoLarsCV object by checking that the optimal alpha
     # increases as the number of samples increases.
@@ -451,16 +445,23 @@ def test_lars_cv():
     assert not hasattr(lars_cv, 'n_nonzero_coefs')
 
 
-@pytest.mark.filterwarnings('ignore::FutureWarning')
-def test_lars_cv_max_iter():
-    with warnings.catch_warnings(record=True) as w:
+def test_lars_cv_max_iter(recwarn):
+    warnings.simplefilter('always')
+    with np.errstate(divide='raise', invalid='raise'):
+        X = diabetes.data
+        y = diabetes.target
         rng = np.random.RandomState(42)
         x = rng.randn(len(y))
         X = diabetes.data
         X = np.c_[X, x, x]  # add correlated features
-        lars_cv = linear_model.LassoLarsCV(max_iter=5)
+        lars_cv = linear_model.LassoLarsCV(max_iter=5, cv=5)
         lars_cv.fit(X, y)
-    assert len(w) == 0
+    # Check that there is no warning in general and no ConvergenceWarning
+    # in particular.
+    # Materialize the string representation of the warning to get a more
+    # informative error message in case of AssertionError.
+    recorded_warnings = [str(w) for w in recwarn]
+    assert recorded_warnings == []
 
 
 def test_lasso_lars_ic():
@@ -477,9 +478,9 @@ def test_lasso_lars_ic():
     lars_aic.fit(X, y)
     nonzero_bic = np.where(lars_bic.coef_)[0]
     nonzero_aic = np.where(lars_aic.coef_)[0]
-    assert_greater(lars_bic.alpha_, lars_aic.alpha_)
-    assert_less(len(nonzero_bic), len(nonzero_aic))
-    assert_less(np.max(nonzero_bic), diabetes.data.shape[1])
+    assert lars_bic.alpha_ > lars_aic.alpha_
+    assert len(nonzero_bic) < len(nonzero_aic)
+    assert np.max(nonzero_bic) < diabetes.data.shape[1]
 
     # test error on unknown IC
     lars_broken = linear_model.LassoLarsIC('<unknown>')
@@ -497,7 +498,6 @@ def test_lars_path_readonly_data():
         _lars_path_residues(X_train, y_train, X_test, y_test, copy=False)
 
 
-@pytest.mark.filterwarnings('ignore: The default of the `iid`')  # 0.22
 def test_lars_path_positive_constraint():
     # this is the main test for the positive parameter on the lars_path method
     # the estimator classes just make use of this function
@@ -508,13 +508,11 @@ def test_lars_path_positive_constraint():
     # and all positive when positive=True
     # for method 'lar' (default) and lasso
 
-    # Once deprecation of LAR + positive option is done use these:
-    # assert_raises(ValueError, linear_model.lars_path, diabetes['data'],
-    #               diabetes['target'], method='lar', positive=True)
-    with pytest.warns(DeprecationWarning, match='broken'):
+    err_msg = "Positive constraint not supported for 'lar' coding method."
+    with pytest.raises(ValueError, match=err_msg):
         linear_model.lars_path(diabetes['data'], diabetes['target'],
-                               return_path=True, method='lar',
-                               positive=True)
+                               method='lar', positive=True)
+
     method = 'lasso'
     _, _, coefs = \
         linear_model.lars_path(X, y, return_path=True, method=method,
@@ -536,7 +534,6 @@ estimator_parameter_map = {'LassoLars': {'alpha': 0.1},
                            'LassoLarsIC': {}}
 
 
-@pytest.mark.filterwarnings('ignore: The default value of cv')  # 0.22
 def test_estimatorclasses_positive_constraint():
     # testing the transmissibility for the positive option of all estimator
     # classes in this same function here
@@ -576,7 +573,7 @@ def test_lasso_lars_vs_lasso_cd_positive():
         lasso_cd.alpha = a
         lasso_cd.fit(X, y)
         error = linalg.norm(c - lasso_cd.coef_)
-        assert_less(error, 0.01)
+        assert error < 0.01
 
     # The range of alphas chosen for coefficient comparison here is restricted
     # as compared with the above test without the positive option. This is due
@@ -593,7 +590,7 @@ def test_lasso_lars_vs_lasso_cd_positive():
         clf2 = linear_model.Lasso(fit_intercept=False, alpha=alpha, tol=1e-8,
                                   normalize=False, positive=True).fit(X, y)
         err = linalg.norm(clf1.coef_ - clf2.coef_)
-        assert_less(err, 1e-3)
+        assert err < 1e-3
 
     # normalized data
     X = diabetes.data
@@ -605,7 +602,7 @@ def test_lasso_lars_vs_lasso_cd_positive():
         lasso_cd.alpha = a
         lasso_cd.fit(X, y)
         error = linalg.norm(c - lasso_cd.coef_)
-        assert_less(error, 0.01)
+        assert error < 0.01
 
 
 def test_lasso_lars_vs_R_implementation():
@@ -690,8 +687,7 @@ def test_lasso_lars_vs_R_implementation():
                    [0, 0, -1.569380717440311, -5.924804108067312,
                     -7.996385265061972]])
 
-    model_lasso_lars2 = linear_model.LassoLars(alpha=0, fit_intercept=True,
-                                               normalize=True)
+    model_lasso_lars2 = linear_model.LassoLars(alpha=0, normalize=True)
     model_lasso_lars2.fit(X, y)
     skl_betas2 = model_lasso_lars2.coef_path_
 
