@@ -1,5 +1,7 @@
 import numpy as np
 
+import pytest
+
 from scipy.sparse import issparse
 from scipy.sparse import coo_matrix
 from scipy.sparse import csc_matrix
@@ -9,20 +11,19 @@ from scipy.sparse import lil_matrix
 
 from sklearn.utils.multiclass import type_of_target
 
-from sklearn.utils.testing import assert_array_equal
-from sklearn.utils.testing import assert_equal
-from sklearn.utils.testing import assert_true
-from sklearn.utils.testing import assert_raises
-from sklearn.utils.testing import assert_raise_message
-from sklearn.utils.testing import ignore_warnings
+from sklearn.utils._testing import assert_array_equal
+from sklearn.utils._testing import assert_warns_message
+from sklearn.utils._testing import ignore_warnings
+from sklearn.utils import _to_object_array
 
-from sklearn.preprocessing.label import LabelBinarizer
-from sklearn.preprocessing.label import MultiLabelBinarizer
-from sklearn.preprocessing.label import LabelEncoder
-from sklearn.preprocessing.label import label_binarize
+from sklearn.preprocessing._label import LabelBinarizer
+from sklearn.preprocessing._label import MultiLabelBinarizer
+from sklearn.preprocessing._label import LabelEncoder
+from sklearn.preprocessing._label import label_binarize
 
-from sklearn.preprocessing.label import _inverse_binarize_thresholding
-from sklearn.preprocessing.label import _inverse_binarize_multiclass
+from sklearn.preprocessing._label import _inverse_binarize_thresholding
+from sklearn.preprocessing._label import _inverse_binarize_multiclass
+from sklearn.preprocessing._label import _encode
 
 from sklearn import datasets
 
@@ -49,7 +50,7 @@ def test_label_binarizer():
     # For sparse case:
     lb = LabelBinarizer(sparse_output=True)
     got = lb.fit_transform(inp)
-    assert_true(issparse(got))
+    assert issparse(got)
     assert_array_equal(lb.classes_, ["pos"])
     assert_array_equal(expected, got.toarray())
     assert_array_equal(lb.inverse_transform(got.toarray()), inp)
@@ -131,45 +132,82 @@ def test_label_binarizer_errors():
     lb = LabelBinarizer().fit(one_class)
 
     multi_label = [(2, 3), (0,), (0, 2)]
-    assert_raises(ValueError, lb.transform, multi_label)
+    with pytest.raises(ValueError):
+        lb.transform(multi_label)
 
     lb = LabelBinarizer()
-    assert_raises(ValueError, lb.transform, [])
-    assert_raises(ValueError, lb.inverse_transform, [])
+    with pytest.raises(ValueError):
+        lb.transform([])
+    with pytest.raises(ValueError):
+        lb.inverse_transform([])
 
-    assert_raises(ValueError, LabelBinarizer, neg_label=2, pos_label=1)
-    assert_raises(ValueError, LabelBinarizer, neg_label=2, pos_label=2)
+    with pytest.raises(ValueError):
+        LabelBinarizer(neg_label=2, pos_label=1)
+    with pytest.raises(ValueError):
+        LabelBinarizer(neg_label=2, pos_label=2)
 
-    assert_raises(ValueError, LabelBinarizer, neg_label=1, pos_label=2,
-                  sparse_output=True)
+    with pytest.raises(ValueError):
+        LabelBinarizer(neg_label=1, pos_label=2, sparse_output=True)
 
     # Fail on y_type
-    assert_raises(ValueError, _inverse_binarize_thresholding,
-                  y=csr_matrix([[1, 2], [2, 1]]), output_type="foo",
-                  classes=[1, 2], threshold=0)
+    with pytest.raises(ValueError):
+        _inverse_binarize_thresholding(y=csr_matrix([[1, 2], [2, 1]]),
+                                       output_type="foo", classes=[1, 2],
+                                       threshold=0)
 
     # Sequence of seq type should raise ValueError
     y_seq_of_seqs = [[], [1, 2], [3], [0, 1, 3], [2]]
-    assert_raises(ValueError, LabelBinarizer().fit_transform, y_seq_of_seqs)
+    with pytest.raises(ValueError):
+        LabelBinarizer().fit_transform(y_seq_of_seqs)
 
     # Fail on the number of classes
-    assert_raises(ValueError, _inverse_binarize_thresholding,
-                  y=csr_matrix([[1, 2], [2, 1]]), output_type="foo",
-                  classes=[1, 2, 3], threshold=0)
+    with pytest.raises(ValueError):
+        _inverse_binarize_thresholding(y=csr_matrix([[1, 2], [2, 1]]),
+                                       output_type="foo",
+                                       classes=[1, 2, 3],
+                                       threshold=0)
 
     # Fail on the dimension of 'binary'
-    assert_raises(ValueError, _inverse_binarize_thresholding,
-                  y=np.array([[1, 2, 3], [2, 1, 3]]), output_type="binary",
-                  classes=[1, 2, 3], threshold=0)
+    with pytest.raises(ValueError):
+        _inverse_binarize_thresholding(y=np.array([[1, 2, 3], [2, 1, 3]]),
+                                       output_type="binary",
+                                       classes=[1, 2, 3],
+                                       threshold=0)
 
     # Fail on multioutput data
-    assert_raises(ValueError, LabelBinarizer().fit, np.array([[1, 3], [2, 1]]))
-    assert_raises(ValueError, label_binarize, np.array([[1, 3], [2, 1]]),
-                  [1, 2, 3])
+    with pytest.raises(ValueError):
+        LabelBinarizer().fit(np.array([[1, 3], [2, 1]]))
+    with pytest.raises(ValueError):
+        label_binarize(np.array([[1, 3], [2, 1]]), [1, 2, 3])
 
 
-def test_label_encoder():
-    # Test LabelEncoder's transform and inverse_transform methods
+@pytest.mark.parametrize(
+        "values, classes, unknown",
+        [(np.array([2, 1, 3, 1, 3], dtype='int64'),
+          np.array([1, 2, 3], dtype='int64'), np.array([4], dtype='int64')),
+         (np.array(['b', 'a', 'c', 'a', 'c'], dtype=object),
+          np.array(['a', 'b', 'c'], dtype=object),
+          np.array(['d'], dtype=object)),
+         (np.array(['b', 'a', 'c', 'a', 'c']),
+          np.array(['a', 'b', 'c']), np.array(['d']))],
+        ids=['int64', 'object', 'str'])
+def test_label_encoder(values, classes, unknown):
+    # Test LabelEncoder's transform, fit_transform and
+    # inverse_transform methods
+    le = LabelEncoder()
+    le.fit(values)
+    assert_array_equal(le.classes_, classes)
+    assert_array_equal(le.transform(values), [1, 0, 2, 0, 2])
+    assert_array_equal(le.inverse_transform([1, 0, 2, 0, 2]), values)
+    le = LabelEncoder()
+    ret = le.fit_transform(values)
+    assert_array_equal(ret, [1, 0, 2, 0, 2])
+
+    with pytest.raises(ValueError, match="unseen labels"):
+        le.transform(unknown)
+
+
+def test_label_encoder_negative_ints():
     le = LabelEncoder()
     le.fit([1, 1, 4, 5, -1, 0])
     assert_array_equal(le.classes_, [-1, 0, 1, 4, 5])
@@ -177,42 +215,65 @@ def test_label_encoder():
                        [1, 2, 3, 3, 4, 0, 0])
     assert_array_equal(le.inverse_transform([1, 2, 3, 3, 4, 0, 0]),
                        [0, 1, 4, 4, 5, -1, -1])
-    assert_raises(ValueError, le.transform, [0, 6])
+    with pytest.raises(ValueError):
+        le.transform([0, 6])
 
-    le.fit(["apple", "orange"])
+
+@pytest.mark.parametrize("dtype", ['str', 'object'])
+def test_label_encoder_str_bad_shape(dtype):
+    le = LabelEncoder()
+    le.fit(np.array(["apple", "orange"], dtype=dtype))
     msg = "bad input shape"
-    assert_raise_message(ValueError, msg, le.transform, "apple")
-
-
-def test_label_encoder_fit_transform():
-    # Test fit_transform
-    le = LabelEncoder()
-    ret = le.fit_transform([1, 1, 4, 5, -1, 0])
-    assert_array_equal(ret, [2, 2, 3, 4, 0, 1])
-
-    le = LabelEncoder()
-    ret = le.fit_transform(["paris", "paris", "tokyo", "amsterdam"])
-    assert_array_equal(ret, [1, 1, 2, 0])
+    with pytest.raises(ValueError, match=msg):
+        le.transform("apple")
 
 
 def test_label_encoder_errors():
     # Check that invalid arguments yield ValueError
     le = LabelEncoder()
-    assert_raises(ValueError, le.transform, [])
-    assert_raises(ValueError, le.inverse_transform, [])
+    with pytest.raises(ValueError):
+        le.transform([])
+    with pytest.raises(ValueError):
+        le.inverse_transform([])
 
     # Fail on unseen labels
     le = LabelEncoder()
-    le.fit([1, 2, 3, 1, -1])
-    assert_raises(ValueError, le.inverse_transform, [-1])
+    le.fit([1, 2, 3, -1, 1])
+    msg = "contains previously unseen labels"
+    with pytest.raises(ValueError, match=msg):
+        le.inverse_transform([-2])
+    with pytest.raises(ValueError, match=msg):
+        le.inverse_transform([-2, -3, -4])
+
+    # Fail on inverse_transform("")
+    msg = "bad input shape ()"
+    with pytest.raises(ValueError, match=msg):
+        le.inverse_transform("")
+
+
+@pytest.mark.parametrize(
+        "values",
+        [np.array([2, 1, 3, 1, 3], dtype='int64'),
+         np.array(['b', 'a', 'c', 'a', 'c'], dtype=object),
+         np.array(['b', 'a', 'c', 'a', 'c'])],
+        ids=['int64', 'object', 'str'])
+def test_label_encoder_empty_array(values):
+    le = LabelEncoder()
+    le.fit(values)
+    # test empty transform
+    transformed = le.transform([])
+    assert_array_equal(np.array([]), transformed)
+    # test empty inverse transform
+    inverse_transformed = le.inverse_transform([])
+    assert_array_equal(np.array([]), inverse_transformed)
 
 
 def test_sparse_output_multilabel_binarizer():
     # test input as iterable of iterables
     inputs = [
         lambda: [(2, 3), (1,), (1, 2)],
-        lambda: (set([2, 3]), set([1]), set([1, 2])),
-        lambda: iter([iter((2, 3)), iter((1,)), set([1, 2])]),
+        lambda: ({2, 3}, {1}, {1, 2}),
+        lambda: iter([iter((2, 3)), iter((1,)), {1, 2}]),
     ]
     indicator_mat = np.array([[0, 1, 1],
                               [1, 0, 0],
@@ -221,57 +282,61 @@ def test_sparse_output_multilabel_binarizer():
     inverse = inputs[0]()
     for sparse_output in [True, False]:
         for inp in inputs:
-            # With fit_tranform
+            # With fit_transform
             mlb = MultiLabelBinarizer(sparse_output=sparse_output)
             got = mlb.fit_transform(inp())
-            assert_equal(issparse(got), sparse_output)
+            assert issparse(got) == sparse_output
             if sparse_output:
+                # verify CSR assumption that indices and indptr have same dtype
+                assert got.indices.dtype == got.indptr.dtype
                 got = got.toarray()
             assert_array_equal(indicator_mat, got)
             assert_array_equal([1, 2, 3], mlb.classes_)
-            assert_equal(mlb.inverse_transform(got), inverse)
+            assert mlb.inverse_transform(got) == inverse
 
             # With fit
             mlb = MultiLabelBinarizer(sparse_output=sparse_output)
             got = mlb.fit(inp()).transform(inp())
-            assert_equal(issparse(got), sparse_output)
+            assert issparse(got) == sparse_output
             if sparse_output:
+                # verify CSR assumption that indices and indptr have same dtype
+                assert got.indices.dtype == got.indptr.dtype
                 got = got.toarray()
             assert_array_equal(indicator_mat, got)
             assert_array_equal([1, 2, 3], mlb.classes_)
-            assert_equal(mlb.inverse_transform(got), inverse)
+            assert mlb.inverse_transform(got) == inverse
 
-    assert_raises(ValueError, mlb.inverse_transform,
-                  csr_matrix(np.array([[0, 1, 1],
-                                       [2, 0, 0],
-                                       [1, 1, 0]])))
+    with pytest.raises(ValueError):
+        mlb.inverse_transform(csr_matrix(np.array([[0, 1, 1],
+                                                   [2, 0, 0],
+                                                   [1, 1, 0]])))
 
 
 def test_multilabel_binarizer():
     # test input as iterable of iterables
     inputs = [
         lambda: [(2, 3), (1,), (1, 2)],
-        lambda: (set([2, 3]), set([1]), set([1, 2])),
-        lambda: iter([iter((2, 3)), iter((1,)), set([1, 2])]),
+        lambda: ({2, 3}, {1}, {1, 2}),
+        lambda: iter([iter((2, 3)), iter((1,)), {1, 2}]),
     ]
     indicator_mat = np.array([[0, 1, 1],
                               [1, 0, 0],
                               [1, 1, 0]])
     inverse = inputs[0]()
     for inp in inputs:
-        # With fit_tranform
+        # With fit_transform
         mlb = MultiLabelBinarizer()
         got = mlb.fit_transform(inp())
         assert_array_equal(indicator_mat, got)
         assert_array_equal([1, 2, 3], mlb.classes_)
-        assert_equal(mlb.inverse_transform(got), inverse)
+        assert mlb.inverse_transform(got) == inverse
 
         # With fit
         mlb = MultiLabelBinarizer()
         got = mlb.fit(inp()).transform(inp())
         assert_array_equal(indicator_mat, got)
         assert_array_equal([1, 2, 3], mlb.classes_)
-        assert_equal(mlb.inverse_transform(got), inverse)
+        assert mlb.inverse_transform(got) == inverse
 
 
 def test_multilabel_binarizer_empty_sample():
@@ -286,10 +351,17 @@ def test_multilabel_binarizer_empty_sample():
 def test_multilabel_binarizer_unknown_class():
     mlb = MultiLabelBinarizer()
     y = [[1, 2]]
-    assert_raises(KeyError, mlb.fit(y).transform, [[0]])
+    Y = np.array([[1, 0], [0, 1]])
+    w = 'unknown class(es) [0, 4] will be ignored'
+    matrix = assert_warns_message(UserWarning, w,
+                                  mlb.fit(y).transform, [[4, 1], [2, 0]])
+    assert_array_equal(matrix, Y)
 
-    mlb = MultiLabelBinarizer(classes=[1, 2])
-    assert_raises(KeyError, mlb.fit_transform, [[0]])
+    Y = np.array([[1, 0, 0], [0, 1, 0]])
+    mlb = MultiLabelBinarizer(classes=[1, 2, 3])
+    matrix = assert_warns_message(UserWarning, w,
+                                  mlb.fit(y).transform, [[4, 1], [2, 0]])
+    assert_array_equal(matrix, Y)
 
 
 def test_multilabel_binarizer_given_classes():
@@ -318,6 +390,31 @@ def test_multilabel_binarizer_given_classes():
     mlb = MultiLabelBinarizer(classes=[1, 3, 2])
     assert_array_equal(mlb.fit(inp).transform(inp), indicator_mat)
 
+    # ensure a ValueError is thrown if given duplicate classes
+    err_msg = "The classes argument contains duplicate classes. Remove " \
+              "these duplicates before passing them to MultiLabelBinarizer."
+    mlb = MultiLabelBinarizer(classes=[1, 3, 2, 3])
+    with pytest.raises(ValueError, match=err_msg):
+        mlb.fit(inp)
+
+
+def test_multilabel_binarizer_multiple_calls():
+    inp = [(2, 3), (1,), (1, 2)]
+    indicator_mat = np.array([[0, 1, 1],
+                              [1, 0, 0],
+                              [1, 0, 1]])
+
+    indicator_mat2 = np.array([[0, 1, 1],
+                               [1, 0, 0],
+                               [1, 1, 0]])
+
+    # first call
+    mlb = MultiLabelBinarizer(classes=[1, 3, 2])
+    assert_array_equal(mlb.fit_transform(inp), indicator_mat)
+    # second call change class
+    mlb.classes = [1, 2, 3]
+    assert_array_equal(mlb.fit_transform(inp), indicator_mat2)
+
 
 def test_multilabel_binarizer_same_length_sequence():
     # Ensure sequences of the same length are not interpreted as a 2-d array
@@ -337,8 +434,7 @@ def test_multilabel_binarizer_same_length_sequence():
 
 
 def test_multilabel_binarizer_non_integer_labels():
-    tuple_classes = np.empty(3, dtype=object)
-    tuple_classes[:] = [(1,), (2,), (3,)]
+    tuple_classes = _to_object_array([(1,), (2,), (3,)])
     inputs = [
         ([('2', '3'), ('1',), ('1', '2')], ['1', '2', '3']),
         ([('b', 'c'), ('a',), ('a', 'b')], ['a', 'b', 'c']),
@@ -361,7 +457,8 @@ def test_multilabel_binarizer_non_integer_labels():
         assert_array_equal(mlb.inverse_transform(indicator_mat), inp)
 
     mlb = MultiLabelBinarizer()
-    assert_raises(TypeError, mlb.fit_transform, [({}), ({}, {'a': 'b'})])
+    with pytest.raises(TypeError):
+        mlb.fit_transform([({}), ({}, {'a': 'b'})])
 
 
 def test_multilabel_binarizer_non_unique():
@@ -376,15 +473,18 @@ def test_multilabel_binarizer_inverse_validation():
     mlb = MultiLabelBinarizer()
     mlb.fit_transform(inp)
     # Not binary
-    assert_raises(ValueError, mlb.inverse_transform, np.array([[1, 3]]))
+    with pytest.raises(ValueError):
+        mlb.inverse_transform(np.array([[1, 3]]))
     # The following binary cases are fine, however
     mlb.inverse_transform(np.array([[0, 0]]))
     mlb.inverse_transform(np.array([[1, 1]]))
     mlb.inverse_transform(np.array([[1, 0]]))
 
     # Wrong shape
-    assert_raises(ValueError, mlb.inverse_transform, np.array([[1]]))
-    assert_raises(ValueError, mlb.inverse_transform, np.array([[1, 1, 1]]))
+    with pytest.raises(ValueError):
+        mlb.inverse_transform(np.array([[1]]))
+    with pytest.raises(ValueError):
+        mlb.inverse_transform(np.array([[1, 1, 1]]))
 
 
 def test_label_binarize_with_class_order():
@@ -408,9 +508,10 @@ def test_label_binarize_with_class_order():
 def check_binarized_results(y, classes, pos_label, neg_label, expected):
     for sparse_output in [True, False]:
         if ((pos_label == 0 or neg_label != 0) and sparse_output):
-            assert_raises(ValueError, label_binarize, y, classes,
-                          neg_label=neg_label, pos_label=pos_label,
-                          sparse_output=sparse_output)
+            with pytest.raises(ValueError):
+                label_binarize(y, classes, neg_label=neg_label,
+                               pos_label=pos_label,
+                               sparse_output=sparse_output)
             continue
 
         # check label_binarize
@@ -418,7 +519,7 @@ def check_binarized_results(y, classes, pos_label, neg_label, expected):
                                    pos_label=pos_label,
                                    sparse_output=sparse_output)
         assert_array_equal(toarray(binarized), expected)
-        assert_equal(issparse(binarized), sparse_output)
+        assert issparse(binarized) == sparse_output
 
         # check inverse
         y_type = type_of_target(y)
@@ -440,10 +541,10 @@ def check_binarized_results(y, classes, pos_label, neg_label, expected):
                             sparse_output=sparse_output)
         binarized = lb.fit_transform(y)
         assert_array_equal(toarray(binarized), expected)
-        assert_equal(issparse(binarized), sparse_output)
+        assert issparse(binarized) == sparse_output
         inverse_output = lb.inverse_transform(binarized)
         assert_array_equal(toarray(inverse_output), toarray(y))
-        assert_equal(issparse(inverse_output), issparse(y))
+        assert issparse(inverse_output) == issparse(y)
 
 
 def test_label_binarize_binary():
@@ -453,7 +554,7 @@ def test_label_binarize_binary():
     neg_label = -1
     expected = np.array([[2, -1], [-1, 2], [2, -1]])[:, 1].reshape((-1, 1))
 
-    yield check_binarized_results, y, classes, pos_label, neg_label, expected
+    check_binarized_results(y, classes, pos_label, neg_label, expected)
 
     # Binary case where sparse_output = True will not result in a ValueError
     y = [0, 1, 0]
@@ -462,7 +563,7 @@ def test_label_binarize_binary():
     neg_label = 0
     expected = np.array([[3, 0], [0, 3], [3, 0]])[:, 1].reshape((-1, 1))
 
-    yield check_binarized_results, y, classes, pos_label, neg_label, expected
+    check_binarized_results(y, classes, pos_label, neg_label, expected)
 
 
 def test_label_binarize_multiclass():
@@ -472,10 +573,11 @@ def test_label_binarize_multiclass():
     neg_label = 0
     expected = 2 * np.eye(3)
 
-    yield check_binarized_results, y, classes, pos_label, neg_label, expected
+    check_binarized_results(y, classes, pos_label, neg_label, expected)
 
-    assert_raises(ValueError, label_binarize, y, classes, neg_label=-1,
-                  pos_label=pos_label, sparse_output=True)
+    with pytest.raises(ValueError):
+        label_binarize(y, classes, neg_label=-1, pos_label=pos_label,
+                       sparse_output=True)
 
 
 def test_label_binarize_multilabel():
@@ -489,16 +591,21 @@ def test_label_binarize_multilabel():
                                       dok_matrix, lil_matrix]]
 
     for y in [y_ind] + y_sparse:
-        yield (check_binarized_results, y, classes, pos_label, neg_label,
-               expected)
+        check_binarized_results(y, classes, pos_label, neg_label,
+                                expected)
 
-    assert_raises(ValueError, label_binarize, y, classes, neg_label=-1,
-                  pos_label=pos_label, sparse_output=True)
+    with pytest.raises(ValueError):
+        label_binarize(y, classes, neg_label=-1, pos_label=pos_label,
+                       sparse_output=True)
 
 
 def test_invalid_input_label_binarize():
-    assert_raises(ValueError, label_binarize, [0, 2], classes=[0, 2],
-                  pos_label=0, neg_label=1)
+    with pytest.raises(ValueError):
+        label_binarize([0, 2], classes=[0, 2], pos_label=0, neg_label=1)
+    with pytest.raises(ValueError, match="continuous target data is not "):
+        label_binarize([1.2, 2.7], classes=[0, 1])
+    with pytest.raises(ValueError, match="mismatch with the labels"):
+        label_binarize([[1, 3]], classes=[1, 2, 3])
 
 
 def test_inverse_binarize_multiclass():
@@ -507,3 +614,43 @@ def test_inverse_binarize_multiclass():
                                                    [0, 0, 0]]),
                                        np.arange(3))
     assert_array_equal(got, np.array([1, 1, 0]))
+
+
+@pytest.mark.parametrize(
+        "values, expected",
+        [(np.array([2, 1, 3, 1, 3], dtype='int64'),
+          np.array([1, 2, 3], dtype='int64')),
+         (np.array(['b', 'a', 'c', 'a', 'c'], dtype=object),
+          np.array(['a', 'b', 'c'], dtype=object)),
+         (np.array(['b', 'a', 'c', 'a', 'c']),
+          np.array(['a', 'b', 'c']))],
+        ids=['int64', 'object', 'str'])
+def test_encode_util(values, expected):
+    uniques = _encode(values)
+    assert_array_equal(uniques, expected)
+    uniques, encoded = _encode(values, encode=True)
+    assert_array_equal(uniques, expected)
+    assert_array_equal(encoded, np.array([1, 0, 2, 0, 2]))
+    _, encoded = _encode(values, uniques, encode=True)
+    assert_array_equal(encoded, np.array([1, 0, 2, 0, 2]))
+
+
+def test_encode_check_unknown():
+    # test for the check_unknown parameter of _encode()
+    uniques = np.array([1, 2, 3])
+    values = np.array([1, 2, 3, 4])
+
+    # Default is True, raise error
+    with pytest.raises(ValueError,
+                       match='y contains previously unseen labels'):
+        _encode(values, uniques, encode=True, check_unknown=True)
+
+    # dont raise error if False
+    _encode(values, uniques, encode=True, check_unknown=False)
+
+    # parameter is ignored for object dtype
+    uniques = np.array(['a', 'b', 'c'], dtype=object)
+    values = np.array(['a', 'b', 'c', 'd'], dtype=object)
+    with pytest.raises(ValueError,
+                       match='y contains previously unseen labels'):
+        _encode(values, uniques, encode=True, check_unknown=False)
